#define INT(x) (x>>8)
#define FIX(x) (x<<8)

struct DynVec
{
	int fx;
	int fy;
	int dx;
	int dy;
};

void Limit(int& x, int min, int max)
{
	if(x<min) x=min;
	if(x>max) x=max;
}

void Decay(int& x, int rate)
{
	if(x>0)
	{
		x-=rate;
		if(x<0) x=0;
	}
	else if(x<0)
	{
		x+=rate;
		if(x>0) x=0;
	}
}

bool BoxBoxIntersect(Box& a, Box& b)
{
	if(a.x+a.w>b.x && b.x+b.w>a.x && a.y+a.h>b.y && b.y+b.h>a.y)
		return true;
	return false;
}

bool PlatformCollide(DynVec& v, Box& box)
{
	if(v.dy<0)
		return false;
	int x=INT(v.fx);
	int y=INT(v.fy);
	for(int i=0;i<maxplatforms;i++)
		if(platforms[i].exists)
		{
			Platform& p=platforms[i];
			if(x+box.w>p.x && x<p.x+p.w && y+box.h<p.y+2)
			{
				if(INT(v.dy)>=p.y-(y+box.h)-2) // would pass through platform in next tick
				{
					v.fy=FIX(p.y-box.h);
					v.dy=0;
					return true;
				}
			}
		}
	return false;
}

bool EntoidHit(Box& ebox, int ttype, Entoid*& target)
{
	target=NULL;
	for(int i=0;i<maxentoids;i++)
		if(entoids[i].exists)
			if(entoids[i].type==ttype && BoxBoxIntersect(ebox, entoids[i].cbox))
			{
				target=&entoids[i];
				return true;
			}
	return false;
}

bool EntoidsHit(Box& ebox, int ttype, Entoid** targets, int& numhit)
{
	numhit=0;
	for(int i=0;i<maxentoids;i++)
		if(entoids[i].exists)
			if(entoids[i].type==ttype && BoxBoxIntersect(ebox, entoids[i].cbox))
			{
				targets[numhit++]=&entoids[i];
//				return true;
			}
	if(numhit>0)
		return true;
	return false;
}

void EntoidMessage(Entoid& recp, int message)
{
	recp.msgn[recp.msgncount++]=message;
}

bool EntoidListen(Entoid& e, int message)
{
	for(int i=0;i<e.msgcount;i++)
		if(e.msg[i]==message)
			return true;
	return false;
}

